
#include <z80retroshield.h>

//
// Our program.
//
unsigned char rom[1024] =
{
  0x31, 0xf9, 0x00, 0x3e, 0x3e, 0xd3, 0x01, 0x21, 0xf9, 0x00, 0x36, 0x0a,
  0xdb, 0x01, 0x77, 0xfe, 0x0a, 0x28, 0x03, 0x23, 0x18, 0xf4, 0x21, 0xf9,
  0x00, 0x7e, 0xfe, 0x63, 0x28, 0x24, 0xfe, 0x43, 0x28, 0x20, 0xfe, 0x64,
  0x28, 0x28, 0xfe, 0x44, 0x28, 0x24, 0xfe, 0x69, 0x28, 0x4b, 0xfe, 0x49,
  0x28, 0x47, 0x3e, 0x45, 0xd3, 0x01, 0x3e, 0x52, 0xd3, 0x01, 0xd3, 0x01,
  0x3e, 0x0a, 0xd3, 0x01, 0x18, 0xc1, 0x21, 0xfa, 0x00, 0xcd, 0xa2, 0x00,
  0x21, 0x03, 0x00, 0xe5, 0xc5, 0xc9, 0x21, 0xfa, 0x00, 0x7e, 0xfe, 0x0a,
  0x28, 0x07, 0xcd, 0xa2, 0x00, 0xed, 0x43, 0xeb, 0x00, 0x2a, 0xeb, 0x00,
  0xcd, 0xce, 0x00, 0x06, 0x10, 0x3e, 0x20, 0xd3, 0x01, 0x4e, 0xcd, 0xd7,
  0x00, 0x23, 0x10, 0xf5, 0x3e, 0x0a, 0xd3, 0x01, 0x22, 0xeb, 0x00, 0x18,
  0x8a, 0x21, 0xfa, 0x00, 0x7e, 0xfe, 0x20, 0x28, 0x07, 0xcd, 0xa2, 0x00,
  0xed, 0x43, 0xed, 0x00, 0x7e, 0x23, 0xfe, 0x20, 0x28, 0xfa, 0xfe, 0x0a,
  0x28, 0xe5, 0x2b, 0xcd, 0xab, 0x00, 0xed, 0x4b, 0xed, 0x00, 0x02, 0x03,
  0xed, 0x43, 0xed, 0x00, 0x18, 0xe6, 0xcd, 0xab, 0x00, 0x47, 0xcd, 0xab,
  0x00, 0x4f, 0xc9, 0x7e, 0xfe, 0x61, 0x38, 0x06, 0xfe, 0x7a, 0x30, 0x02,
  0xd6, 0x20, 0xcd, 0xc6, 0x00, 0x87, 0x87, 0x87, 0x87, 0x57, 0x23, 0x7e,
  0xcd, 0xc6, 0x00, 0xb2, 0x23, 0xc9, 0xd6, 0x30, 0xfe, 0x0a, 0xd8, 0xd6,
  0x07, 0xc9, 0x4c, 0xcd, 0xd7, 0x00, 0x4d, 0xcd, 0xd7, 0x00, 0xc9, 0x79,
  0x1f, 0x1f, 0x1f, 0x1f, 0xcd, 0xe0, 0x00, 0x79, 0xe6, 0x0f, 0xc6, 0x90,
  0x27, 0xce, 0x40, 0x27, 0xd3, 0x01, 0xc9, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};
int rom_len = sizeof(rom) / sizeof(rom[0]);


//
// Our helper
//
Z80RetroShield cpu;


//
// RAM I/O function handler.
//
char ram_read(int address)
{
    return (rom[address]) ;
}

//
// RAM I/O function handler.
//
void ram_write(int address, byte value)
{
    rom[address] = value;
}


//
// I/O function handler.
//
char io_read(int address)
{
    if (address == 1)
    {
        while (! Serial.available())
        {
            delay(1);
        }

        char r = Serial.read();
        Serial.print( "I/O READ " );
        Serial.print( address, HEX );
        Serial.print( " -> " );
        Serial.println( r, HEX );
        return (r);
    }

    return 0;
}

//
// I/O function handler.
//
void io_write(int address, char byte)
{
    if (address == 1)
    {
        Serial.write(byte);
        return;
    }

    return;
}


//
// Setup routine: Called once.
//
void setup()
{
    Serial.begin(115200);


    //
    // Setup callbacks.
    //
    // We have to setup a RAM-read callback, otherwise the program
    // won't be fetched from RAM and executed.
    //
    // Then we setup a callback to be executed every time an IN/OUT
    // instruction is encountered.
    //
    cpu.set_ram_read(ram_read);
    cpu.set_ram_write(ram_write);
    cpu.set_io_read(io_read);
    cpu.set_io_write(io_write);

    //
    // Configured.
    //
    Serial.println("Z80 configured; launching program.");
}


//
// Loop function: Called forever.
//
void loop()
{
    //
    // Step the CPU.
    //
    cpu.Tick();

}
